!Copyright 2017 R. CARPENTIER, P. CORDE, F. JEZEQUEL, J.-L. LAMOTTE
!
!This file is part of CADNA.
!
!    CADNA is free software: you can redistribute it and/or modify it
!    under the terms of the GNU Lesser General Public License as
!    published by the Free Software Foundation, either version 3 of the
!    License, or (at your option) any later version.
!
!    CADNA is distributed in the hope that it will be useful, but WITHOUT
!    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
!    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
!    Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public
!    License along with CADNA. If not, see
!    <http://www.gnu.org/licenses/>.
!

! Module de conversion




module cadna_convert
  use ISO_C_BINDING, only : C_INT, C_FLOAT, C_DOUBLE
  use cadna_types
  use cadna_affect

  implicit none




  interface int
    module procedure int_single_st
  end interface
  interface
    pure function cpp_int_single_st(a) bind(C)
      import single_st
      import C_INT 
      type(single_st), intent(in) :: a
      integer(C_INT) cpp_int_single_st
    end function cpp_int_single_st
  end interface

  interface int
    module procedure int_double_st
  end interface
  interface
    pure function cpp_int_double_st(a) bind(C)
      import double_st
      import C_INT 
      type(double_st), intent(in) :: a
      integer(C_INT) cpp_int_double_st
    end function cpp_int_double_st
  end interface

  interface real
    module procedure real_single_st
  end interface
  interface
    pure function cpp_real_single_st(a) bind(C)
      import single_st
      import C_FLOAT 
      type(single_st), intent(in) :: a
      real(C_FLOAT) cpp_real_single_st
    end function cpp_real_single_st
  end interface

  interface real
    module procedure real_double_st
  end interface
  interface
    pure function cpp_real_double_st(a) bind(C)
      import double_st
      import C_FLOAT 
      type(double_st), intent(in) :: a
      real(C_FLOAT) cpp_real_double_st
    end function cpp_real_double_st
  end interface

  interface dble
    module procedure dble_single_st
  end interface
  interface
    pure function cpp_dble_single_st(a) bind(C)
      import single_st
      import C_DOUBLE 
      type(single_st), intent(in) :: a
      real(C_DOUBLE) cpp_dble_single_st
    end function cpp_dble_single_st
  end interface

  interface dble
    module procedure dble_double_st
  end interface
  interface
    pure function cpp_dble_double_st(a) bind(C)
      import double_st
      import C_DOUBLE 
      type(double_st), intent(in) :: a
      real(C_DOUBLE) cpp_dble_double_st
    end function cpp_dble_double_st
  end interface


contains



  elemental function int_single_st(a)
    type(single_st), intent(in) :: a
    integer(C_INT) int_single_st

    int_single_st = cpp_int_single_st(a)
  end function int_single_st

  elemental function int_double_st(a)
    type(double_st), intent(in) :: a
    integer(C_INT) int_double_st

    int_double_st = cpp_int_double_st(a)
  end function int_double_st

  elemental function real_single_st(a)
    type(single_st), intent(in) :: a
    real(C_FLOAT) real_single_st

    real_single_st = cpp_real_single_st(a)
  end function real_single_st

  elemental function real_double_st(a)
    type(double_st), intent(in) :: a
    real(C_FLOAT) real_double_st

    real_double_st = cpp_real_double_st(a)
  end function real_double_st

  elemental function dble_single_st(a)
    type(single_st), intent(in) :: a
    real(C_DOUBLE) dble_single_st

    dble_single_st = cpp_dble_single_st(a)
  end function dble_single_st

  elemental function dble_double_st(a)
    type(double_st), intent(in) :: a
    real(C_DOUBLE) dble_double_st

    dble_double_st = cpp_dble_double_st(a)
  end function dble_double_st




end module cadna_convert

